@@ -26,6 +26,7 @@
 #include <arrow-glib/record-batch.hpp>
 #include <arrow-glib/schema.hpp>
 
+#include <arrow/c/bridge.h>
 #include <arrow/util/iterator.h>
 
 #include <sstream>
@@ -48,7 +49,8 @@ G_BEGIN_DECLS
  * batches.
  */
 
-typedef struct GArrowRecordBatchPrivate_ {
+typedef struct GArrowRecordBatchPrivate_
+{
   std::shared_ptr<arrow::RecordBatch> record_batch;
 } GArrowRecordBatchPrivate;
 
@@ -56,14 +58,11 @@ enum {
   PROP_RECORD_BATCH = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GArrowRecordBatch,
-                           garrow_record_batch,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GArrowRecordBatch, garrow_record_batch, G_TYPE_OBJECT)
 
-#define GARROW_RECORD_BATCH_GET_PRIVATE(obj)         \
-  static_cast<GArrowRecordBatchPrivate *>(           \
-     garrow_record_batch_get_instance_private(       \
-       GARROW_RECORD_BATCH(obj)))
+#define GARROW_RECORD_BATCH_GET_PRIVATE(obj)                                             \
+  static_cast<GArrowRecordBatchPrivate *>(                                               \
+    garrow_record_batch_get_instance_private(GARROW_RECORD_BATCH(obj)))
 
 static void
 garrow_record_batch_finalize(GObject *object)
@@ -111,7 +110,7 @@ static void
 garrow_record_batch_init(GArrowRecordBatch *object)
 {
   auto priv = GARROW_RECORD_BATCH_GET_PRIVATE(object);
-  new(&priv->record_batch) std::shared_ptr<arrow::RecordBatch>;
+  new (&priv->record_batch) std::shared_ptr<arrow::RecordBatch>;
 }
 
 static void
@@ -122,18 +121,45 @@ garrow_record_batch_class_init(GArrowRecordBatchClass *klass)
 
   gobject_class = G_OBJECT_CLASS(klass);
 
-  gobject_class->finalize     = garrow_record_batch_finalize;
+  gobject_class->finalize = garrow_record_batch_finalize;
   gobject_class->set_property = garrow_record_batch_set_property;
   gobject_class->get_property = garrow_record_batch_get_property;
 
-  spec = g_param_spec_pointer("record-batch",
-                              "RecordBatch",
-                              "The raw std::shared<arrow::RecordBatch> *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "record-batch",
+    "RecordBatch",
+    "The raw std::shared<arrow::RecordBatch> *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_RECORD_BATCH, spec);
 }
 
+/**
+ * garrow_record_batch_import:
+ * @c_abi_array: (not nullable): A `struct ArrowArray *`.
+ * @schema: A #GArrowSchema of the C ABI array.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (transfer full) (nullable): An imported #GArrowRecordBatch
+ *   on success, %NULL on error.
+ *
+ *   You don't need to release the passed `struct ArrowArray *`,
+ *   even if this function reports an error.
+ *
+ * Since: 6.0.0
+ */
+GArrowRecordBatch *
+garrow_record_batch_import(gpointer c_abi_array, GArrowSchema *schema, GError **error)
+{
+  auto arrow_schema = garrow_schema_get_raw(schema);
+  auto arrow_record_batch_result =
+    arrow::ImportRecordBatch(static_cast<ArrowArray *>(c_abi_array), arrow_schema);
+  if (garrow::check(error, arrow_record_batch_result, "[record-batch][import]")) {
+    return garrow_record_batch_new_raw(&(*arrow_record_batch_result));
+  } else {
+    return NULL;
+  }
+}
+
 /**
  * garrow_record_batch_new:
  * @schema: The schema of the record batch.
@@ -159,19 +185,59 @@ garrow_record_batch_new(GArrowSchema *schema,
 
   const auto &arrow_schema = garrow_schema_get_raw(schema);
   if (arrow_schema->num_fields() != static_cast<int>(arrow_columns.size())) {
-    auto status =
-      arrow::Status::Invalid("Number of columns did not match schema");
+    auto status = arrow::Status::Invalid("Number of columns did not match schema");
     garrow_error_check(error, status, tag);
     return NULL;
   }
 
-  auto arrow_record_batch =
-    arrow::RecordBatch::Make(arrow_schema, n_rows, arrow_columns);
-  auto status = arrow_record_batch->Validate();
-  if (garrow_error_check(error, status, tag)) {
-    return garrow_record_batch_new_raw(&arrow_record_batch);
+  auto arrow_record_batch = arrow::RecordBatch::Make(arrow_schema, n_rows, arrow_columns);
+  return garrow_record_batch_new_raw(&arrow_record_batch);
+}
+
+/**
+ * garrow_record_batch_export:
+ * @record_batch: A #GArrowRecordBatch.
+ * @c_abi_array: (out): Return location for a `struct ArrowArray *`.
+ *   It should be freed with the `ArrowArray::release` callback then
+ *   g_free() when no longer needed.
+ * @c_abi_schema: (out) (nullable): Return location for a
+ *   `struct ArrowSchema *` or %NULL.
+ *   It should be freed with the `ArrowSchema::release` callback then
+ *   g_free() when no longer needed.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 6.0.0
+ */
+gboolean
+garrow_record_batch_export(GArrowRecordBatch *record_batch,
+                           gpointer *c_abi_array,
+                           gpointer *c_abi_schema,
+                           GError **error)
+{
+  const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
+  *c_abi_array = g_new(ArrowArray, 1);
+  arrow::Status status;
+  if (c_abi_schema) {
+    *c_abi_schema = g_new(ArrowSchema, 1);
+    status = arrow::ExportRecordBatch(*arrow_record_batch,
+                                      static_cast<ArrowArray *>(*c_abi_array),
+                                      static_cast<ArrowSchema *>(*c_abi_schema));
   } else {
-    return NULL;
+    status = arrow::ExportRecordBatch(*arrow_record_batch,
+                                      static_cast<ArrowArray *>(*c_abi_array));
+  }
+  if (garrow::check(error, status, "[record-batch][export]")) {
+    return true;
+  } else {
+    g_free(*c_abi_array);
+    *c_abi_array = nullptr;
+    if (c_abi_schema) {
+      g_free(*c_abi_schema);
+      *c_abi_schema = nullptr;
+    }
+    return false;
   }
 }
 
@@ -190,8 +256,7 @@ garrow_record_batch_equal(GArrowRecordBatch *record_batch,
                           GArrowRecordBatch *other_record_batch)
 {
   const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
-  const auto arrow_other_record_batch =
-    garrow_record_batch_get_raw(other_record_batch);
+  const auto arrow_other_record_batch = garrow_record_batch_get_raw(other_record_batch);
   return arrow_record_batch->Equals(*arrow_other_record_batch);
 }
 
@@ -243,8 +308,7 @@ garrow_record_batch_get_schema(GArrowRecordBatch *record_batch)
  * Since: 0.15.0
  */
 GArrowArray *
-garrow_record_batch_get_column_data(GArrowRecordBatch *record_batch,
-                                    gint i)
+garrow_record_batch_get_column_data(GArrowRecordBatch *record_batch, gint i)
 {
   const auto &arrow_record_batch = garrow_record_batch_get_raw(record_batch);
   if (!garrow_internal_index_adjust(i, arrow_record_batch->num_columns())) {
@@ -265,8 +329,7 @@ garrow_record_batch_get_column_data(GArrowRecordBatch *record_batch,
  *   on success, %NULL on out of index
  */
 const gchar *
-garrow_record_batch_get_column_name(GArrowRecordBatch *record_batch,
-                                    gint i)
+garrow_record_batch_get_column_name(GArrowRecordBatch *record_batch, gint i)
 {
   const auto &arrow_record_batch = garrow_record_batch_get_raw(record_batch);
   if (!garrow_internal_index_adjust(i, arrow_record_batch->num_columns())) {
@@ -313,9 +376,7 @@ garrow_record_batch_get_n_rows(GArrowRecordBatch *record_batch)
  *   #GArrowRecordBatch.
  */
 GArrowRecordBatch *
-garrow_record_batch_slice(GArrowRecordBatch *record_batch,
-                          gint64 offset,
-                          gint64 length)
+garrow_record_batch_slice(GArrowRecordBatch *record_batch, gint64 offset, gint64 length)
 {
   const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
   auto arrow_sub_record_batch = arrow_record_batch->Slice(offset, length);
@@ -338,7 +399,8 @@ gchar *
 garrow_record_batch_to_string(GArrowRecordBatch *record_batch, GError **error)
 {
   const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
-  return g_strdup(arrow_record_batch->ToString().c_str());
+  const auto string = arrow_record_batch->ToString();
+  return g_strdup(string.c_str());
 }
 
 /**
@@ -366,9 +428,7 @@ garrow_record_batch_add_column(GArrowRecordBatch *record_batch,
   const auto arrow_column = garrow_array_get_raw(column);
   auto arrow_new_record_batch =
     arrow_record_batch->AddColumn(i, arrow_field, arrow_column);
-  if (garrow::check(error,
-                    arrow_new_record_batch,
-                    "[record-batch][add-column]")) {
+  if (garrow::check(error, arrow_new_record_batch, "[record-batch][add-column]")) {
     return garrow_record_batch_new_raw(&(*arrow_new_record_batch));
   } else {
     return NULL;
@@ -393,9 +453,7 @@ garrow_record_batch_remove_column(GArrowRecordBatch *record_batch,
 {
   const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
   auto arrow_new_record_batch = arrow_record_batch->RemoveColumn(i);
-  if (garrow::check(error,
-                    arrow_new_record_batch,
-                    "[record-batch][remove-column]")) {
+  if (garrow::check(error, arrow_new_record_batch, "[record-batch][remove-column]")) {
     return garrow_record_batch_new_raw(&(*arrow_new_record_batch));
   } else {
     return NULL;
@@ -423,8 +481,8 @@ garrow_record_batch_serialize(GArrowRecordBatch *record_batch,
   arrow::Result<std::shared_ptr<arrow::Buffer>> arrow_buffer;
   if (options) {
     auto arrow_options = garrow_write_options_get_raw(options);
-    auto arrow_buffer = arrow::ipc::SerializeRecordBatch(*arrow_record_batch,
-                                                         *arrow_options);
+    auto arrow_buffer =
+      arrow::ipc::SerializeRecordBatch(*arrow_record_batch, *arrow_options);
     if (garrow::check(error, arrow_buffer, "[record-batch][serialize]")) {
       return garrow_buffer_new_raw(&(*arrow_buffer));
     } else {
@@ -432,8 +490,8 @@ garrow_record_batch_serialize(GArrowRecordBatch *record_batch,
     }
   } else {
     const auto arrow_options = arrow::ipc::IpcWriteOptions::Defaults();
-    auto arrow_buffer = arrow::ipc::SerializeRecordBatch(*arrow_record_batch,
-                                                         arrow_options);
+    auto arrow_buffer =
+      arrow::ipc::SerializeRecordBatch(*arrow_record_batch, arrow_options);
     if (garrow::check(error, arrow_buffer, "[record-batch][serialize]")) {
       return garrow_buffer_new_raw(&(*arrow_buffer));
     } else {
@@ -442,8 +500,42 @@ garrow_record_batch_serialize(GArrowRecordBatch *record_batch,
   }
 }
 
+/**
+ * garrow_record_batch_validate
+ * @record_batch: A #GArrowRecordBatch
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 20.0.0
+ */
+gboolean
+garrow_record_batch_validate(GArrowRecordBatch *record_batch, GError **error)
+{
+  const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
+  return garrow::check(error, arrow_record_batch->Validate(), "[record-batch][validate]");
+}
 
-typedef struct GArrowRecordBatchIteratorPrivate_ {
+/**
+ * garrow_record_batch_validate_full
+ * @record_batch: A #GArrowRecordBatch
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 20.0.0
+ */
+gboolean
+garrow_record_batch_validate_full(GArrowRecordBatch *record_batch, GError **error)
+{
+  const auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
+  return garrow::check(error,
+                       arrow_record_batch->ValidateFull(),
+                       "[record-batch][validate-full]");
+}
+
+typedef struct GArrowRecordBatchIteratorPrivate_
+{
   arrow::RecordBatchIterator iterator;
 } GArrowRecordBatchIteratorPrivate;
 
@@ -455,10 +547,10 @@ G_DEFINE_TYPE_WITH_PRIVATE(GArrowRecordBatchIterator,
                            garrow_record_batch_iterator,
                            G_TYPE_OBJECT)
 
-#define GARROW_RECORD_BATCH_ITERATOR_GET_PRIVATE(obj)        \
-  static_cast<GArrowRecordBatchIteratorPrivate *>(           \
-     garrow_record_batch_iterator_get_instance_private(      \
-       GARROW_RECORD_BATCH_ITERATOR(obj)))
+#define GARROW_RECORD_BATCH_ITERATOR_GET_PRIVATE(obj)                                    \
+  static_cast<GArrowRecordBatchIteratorPrivate *>(                                       \
+    garrow_record_batch_iterator_get_instance_private(                                   \
+      GARROW_RECORD_BATCH_ITERATOR(obj)))
 
 static void
 garrow_record_batch_iterator_finalize(GObject *object)
@@ -493,7 +585,7 @@ static void
 garrow_record_batch_iterator_init(GArrowRecordBatchIterator *object)
 {
   auto priv = GARROW_RECORD_BATCH_ITERATOR_GET_PRIVATE(object);
-  new(&priv->iterator) arrow::RecordBatchIterator;
+  new (&priv->iterator) arrow::RecordBatchIterator;
 }
 
 static void
@@ -501,16 +593,16 @@ garrow_record_batch_iterator_class_init(GArrowRecordBatchIteratorClass *klass)
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
 
-  gobject_class->finalize     = garrow_record_batch_iterator_finalize;
+  gobject_class->finalize = garrow_record_batch_iterator_finalize;
   gobject_class->set_property = garrow_record_batch_iterator_set_property;
 
   GParamSpec *spec;
 
-  spec = g_param_spec_pointer("iterator",
-                              "Iterator",
-                              "The raw arrow::RecordBatchIterator",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "iterator",
+    "Iterator",
+    "The raw arrow::RecordBatchIterator",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_ITERATOR, spec);
 }
 
@@ -547,8 +639,7 @@ garrow_record_batch_iterator_new(GList *record_batches)
  * Since: 0.17.0
  */
 GArrowRecordBatch *
-garrow_record_batch_iterator_next(GArrowRecordBatchIterator *iterator,
-                                  GError **error)
+garrow_record_batch_iterator_next(GArrowRecordBatchIterator *iterator, GError **error)
 {
   auto priv = GARROW_RECORD_BATCH_ITERATOR_GET_PRIVATE(iterator);
 
@@ -590,9 +681,8 @@ garrow_record_batch_iterator_equal(GArrowRecordBatchIterator *iterator,
  *
  * Since: 0.17.0
  */
-GList*
-garrow_record_batch_iterator_to_list(GArrowRecordBatchIterator *iterator,
-                                     GError **error)
+GList *
+garrow_record_batch_iterator_to_list(GArrowRecordBatchIterator *iterator, GError **error)
 {
   auto priv = GARROW_RECORD_BATCH_ITERATOR_GET_PRIVATE(iterator);
   GList *record_batches = NULL;
@@ -615,10 +705,8 @@ G_END_DECLS
 GArrowRecordBatch *
 garrow_record_batch_new_raw(std::shared_ptr<arrow::RecordBatch> *arrow_record_batch)
 {
-  auto record_batch =
-    GARROW_RECORD_BATCH(g_object_new(GARROW_TYPE_RECORD_BATCH,
-                                     "record-batch", arrow_record_batch,
-                                     NULL));
+  auto record_batch = GARROW_RECORD_BATCH(
+    g_object_new(GARROW_TYPE_RECORD_BATCH, "record-batch", arrow_record_batch, NULL));
   return record_batch;
 }
 
@@ -632,9 +720,8 @@ garrow_record_batch_get_raw(GArrowRecordBatch *record_batch)
 GArrowRecordBatchIterator *
 garrow_record_batch_iterator_new_raw(arrow::RecordBatchIterator *arrow_iterator)
 {
-  auto iterator = g_object_new(GARROW_TYPE_RECORD_BATCH_ITERATOR,
-                               "iterator", arrow_iterator,
-                               NULL);
+  auto iterator =
+    g_object_new(GARROW_TYPE_RECORD_BATCH_ITERATOR, "iterator", arrow_iterator, NULL);
   return GARROW_RECORD_BATCH_ITERATOR(iterator);
 }
 
