@@ -15,20 +15,22 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include "arrow/csv/converter.h"
+
+#include <gtest/gtest.h>
+
 #include <cstdint>
 #include <memory>
 #include <set>
 #include <string>
 #include <vector>
 
-#include <gtest/gtest.h>
-
 #include "arrow/array.h"
 #include "arrow/array/builder_decimal.h"
-#include "arrow/csv/converter.h"
 #include "arrow/csv/options.h"
 #include "arrow/csv/test_common.h"
 #include "arrow/status.h"
+#include "arrow/testing/builder.h"
 #include "arrow/testing/gtest_util.h"
 #include "arrow/type.h"
 #include "arrow/type_traits.h"
@@ -173,67 +175,105 @@ void AssertConversionError(const std::shared_ptr<DataType>& type,
 // Converter tests
 
 template <typename T>
-static void TestBinaryConversionBasics() {
-  auto type = TypeTraits<T>::type_singleton();
-  AssertConversion<T, std::string>(type, {"ab,cdé\n", ",\xffgh\n"},
-                                   {{"ab", ""}, {"cdé", "\xffgh"}});
-}
+class BinaryConversionTestBase : public testing::Test {
+ public:
+  std::shared_ptr<DataType> type() { return TypeTraits<T>::type_singleton(); }
 
-TEST(BinaryConversion, Basics) { TestBinaryConversionBasics<BinaryType>(); }
+  void TestNulls() {
+    auto type = this->type();
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "NULL,\n"},
+                                     {{"ab", "NULL"}, {"N/A", ""}},
+                                     {{true, true}, {true, true}});
 
-TEST(LargeBinaryConversion, Basics) { TestBinaryConversionBasics<LargeBinaryType>(); }
+    auto options = ConvertOptions::Defaults();
+    options.strings_can_be_null = true;
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "NULL,\n"},
+                                     {{"ab", ""}, {"", ""}},
+                                     {{true, false}, {false, false}}, options);
+    AssertConversion<T, std::string>(type, {"ab,\"N/A\"\n", "\"NULL\",\"\"\n"},
+                                     {{"ab", ""}, {"", ""}},
+                                     {{true, false}, {false, false}}, options);
+    options.quoted_strings_can_be_null = false;
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "NULL,\n"},
+                                     {{"ab", ""}, {"", ""}},
+                                     {{true, false}, {false, false}}, options);
+    AssertConversion<T, std::string>(type, {"ab,\"N/A\"\n", "\"NULL\",\"\"\n"},
+                                     {{"ab", "NULL"}, {"N/A", ""}},
+                                     {{true, true}, {true, true}}, options);
+  }
 
-TEST(BinaryConversion, Nulls) {
-  AssertConversion<BinaryType, std::string>(binary(), {"ab,N/A\n", "NULL,\n"},
-                                            {{"ab", "NULL"}, {"N/A", ""}},
-                                            {{true, true}, {true, true}});
+  void TestCustomNulls() {
+    auto type = this->type();
+    auto options = ConvertOptions::Defaults();
+    options.null_values = {"xxx", "zzz"};
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "xxx,\"zzz\"\n"},
+                                     {{"ab", "xxx"}, {"N/A", "zzz"}},
+                                     {{true, true}, {true, true}}, options);
 
-  auto options = ConvertOptions::Defaults();
-  options.strings_can_be_null = true;
-  AssertConversion<BinaryType, std::string>(binary(), {"ab,N/A\n", "NULL,\n"},
-                                            {{"ab", ""}, {"", ""}},
-                                            {{true, false}, {false, false}}, options);
-}
+    options.strings_can_be_null = true;
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "xxx,\"zzz\"\n"},
+                                     {{"ab", ""}, {"N/A", ""}},
+                                     {{true, false}, {true, false}}, options);
+    options.quoted_strings_can_be_null = false;
+    AssertConversion<T, std::string>(type, {"ab,N/A\n", "xxx,\"zzz\"\n"},
+                                     {{"ab", ""}, {"N/A", "zzz"}},
+                                     {{true, false}, {true, true}}, options);
+  }
+};
 
 template <typename T>
-static void TestStringConversionBasics() {
-  auto type = TypeTraits<T>::type_singleton();
-  AssertConversion<T, std::string>(type, {"ab,cdé\n", ",gh\n"},
-                                   {{"ab", ""}, {"cdé", "gh"}});
+class BinaryConversionTest : public BinaryConversionTestBase<T> {
+ public:
+  void TestBasics() {
+    auto type = this->type();
+    AssertConversion<T, std::string>(type, {"ab,cdé\n", ",\xffgh\n"},
+                                     {{"ab", ""}, {"cdé", "\xffgh"}});
+  }
+};
 
-  auto options = ConvertOptions::Defaults();
-  options.check_utf8 = false;
-  AssertConversion<T, std::string>(type, {"ab,cdé\n", ",\xffgh\n"},
-                                   {{"ab", ""}, {"cdé", "\xffgh"}}, options,
-                                   /*validate_full=*/false);
-}
+using BinaryTestTypes = ::testing::Types<BinaryType, LargeBinaryType>;
 
-TEST(StringConversion, Basics) { TestStringConversionBasics<StringType>(); }
+TYPED_TEST_SUITE(BinaryConversionTest, BinaryTestTypes);
 
-TEST(LargeStringConversion, Basics) { TestStringConversionBasics<LargeStringType>(); }
+TYPED_TEST(BinaryConversionTest, Basics) { this->TestBasics(); }
 
-TEST(StringConversion, Nulls) {
-  AssertConversion<StringType, std::string>(utf8(), {"ab,N/A\n", "NULL,\n"},
-                                            {{"ab", "NULL"}, {"N/A", ""}},
-                                            {{true, true}, {true, true}});
+TYPED_TEST(BinaryConversionTest, Nulls) { this->TestNulls(); }
 
-  auto options = ConvertOptions::Defaults();
-  options.strings_can_be_null = true;
-  AssertConversion<StringType, std::string>(utf8(), {"ab,N/A\n", "NULL,\n"},
-                                            {{"ab", ""}, {"", ""}},
-                                            {{true, false}, {false, false}}, options);
-}
+TYPED_TEST(BinaryConversionTest, CustomNulls) { this->TestNulls(); }
 
 template <typename T>
-static void TestStringConversionErrors() {
-  auto type = TypeTraits<T>::type_singleton();
-  // Invalid UTF8 in column 0
-  AssertConversionError(type, {"ab,cdé\n", "\xff,gh\n"}, {0});
-}
+class StringConversionTest : public BinaryConversionTestBase<T> {
+ public:
+  void TestBasics() {
+    auto type = TypeTraits<T>::type_singleton();
+    AssertConversion<T, std::string>(type, {"ab,cdé\n", ",gh\n"},
+                                     {{"ab", ""}, {"cdé", "gh"}});
+  }
 
-TEST(StringConversion, Errors) { TestStringConversionErrors<StringType>(); }
+  void TestInvalidUtf8() {
+    auto type = TypeTraits<T>::type_singleton();
+    // Invalid UTF8 in column 0
+    AssertConversionError(type, {"ab,cdé\n", "\xff,gh\n"}, {0});
 
-TEST(LargeStringConversion, Errors) { TestStringConversionErrors<LargeStringType>(); }
+    auto options = ConvertOptions::Defaults();
+    options.check_utf8 = false;
+    AssertConversion<T, std::string>(type, {"ab,cdé\n", ",\xffgh\n"},
+                                     {{"ab", ""}, {"cdé", "\xffgh"}}, options,
+                                     /*validate_full=*/false);
+  }
+};
+
+using StringTestTypes = ::testing::Types<StringType, LargeStringType>;
+
+TYPED_TEST_SUITE(StringConversionTest, StringTestTypes);
+
+TYPED_TEST(StringConversionTest, Basics) { this->TestBasics(); }
+
+TYPED_TEST(StringConversionTest, Nulls) { this->TestNulls(); }
+
+TYPED_TEST(StringConversionTest, CustomNulls) { this->TestCustomNulls(); }
+
+TYPED_TEST(StringConversionTest, InvalidUtf8) { this->TestInvalidUtf8(); }
 
 TEST(FixedSizeBinaryConversion, Basics) {
   AssertConversion<FixedSizeBinaryType, std::string>(
@@ -245,6 +285,38 @@ TEST(FixedSizeBinaryConversion, Errors) {
   AssertConversionError(fixed_size_binary(2), {"ab,cd\n", "g,ij\n"}, {0});
 }
 
+TEST(FixedSizeBinaryConversion, Nulls) {
+  AssertConversion<FixedSizeBinaryType, std::string>(
+      fixed_size_binary(2), {"ab,N/A\n", ",ij\n"}, {{"ab", "\0\0"}, {"\0\0", "ij"}},
+      {{true, false}, {false, true}});
+
+  AssertConversionAllNulls<FixedSizeBinaryType, std::string>(fixed_size_binary(2));
+}
+
+TEST(FixedSizeBinaryConversion, CustomNulls) {
+  auto options = ConvertOptions::Defaults();
+  options.null_values = {"xxx", "zzz"};
+
+  AssertConversion<FixedSizeBinaryType, std::string>(fixed_size_binary(2),
+                                                     {"\"ab\",\"xxx\"\n"}, {{"ab"}, {""}},
+                                                     {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(fixed_size_binary(2), {"\"ab\",\"xxx\"\n"}, {1}, options);
+
+  AssertConversion<FixedSizeBinaryType, std::string>(
+      fixed_size_binary(2), {"ab,xxx\n", "zzz,ij\n"}, {{"ab", "\0\0"}, {"\0\0", "ij"}},
+      {{true, false}, {false, true}}, options);
+
+  AssertConversionError(fixed_size_binary(2), {",xxx,N/A\n"}, {0, 2}, options);
+
+  // Duplicate nulls allowed
+  options.null_values = {"xxx", "zzz", "xxx"};
+  AssertConversion<FixedSizeBinaryType, std::string>(
+      fixed_size_binary(2), {"ab,xxx\n", "zzz,ij\n"}, {{"ab", "\0,\0"}, {"\0\0", "ij"}},
+      {{true, false}, {false, true}}, options);
+}
+
 TEST(NullConversion, Basics) {
   std::shared_ptr<BlockParser> parser;
   std::shared_ptr<Converter> converter;
@@ -287,6 +359,12 @@ TEST(IntegerConversion, CustomNulls) {
   auto options = ConvertOptions::Defaults();
   options.null_values = {"xxx", "zzz"};
 
+  AssertConversion<Int8Type, int8_t>(int8(), {"\"12\",\"xxx\"\n"}, {{12}, {0}},
+                                     {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(int8(), {"\"12\",\"xxx\"\n"}, {1}, options);
+
   AssertConversion<Int8Type, int8_t>(int8(), {"12,xxx\n", "zzz,-128\n"},
                                      {{12, 0}, {0, -128}}, {{true, false}, {false, true}},
                                      options);
@@ -324,6 +402,12 @@ TEST(FloatingPointConversion, CustomNulls) {
   auto options = ConvertOptions::Defaults();
   options.null_values = {"xxx", "zzz"};
 
+  AssertConversion<FloatType, float>(float32(), {"\"1.5\",\"xxx\"\n"}, {{1.5}, {0}},
+                                     {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(float32(), {"\"1.5\",\"xxx\"\n"}, {1}, options);
+
   AssertConversion<FloatType, float>(float32(), {"1.5,xxx\n", "zzz,-1e10\n"},
                                      {{1.5, 0.}, {0., -1e10f}},
                                      {{true, false}, {false, true}}, options);
@@ -334,6 +418,18 @@ TEST(FloatingPointConversion, Whitespace) {
                                        {{12., 0.}, {34.5, -1e100}});
 }
 
+TEST(FloatingPointConversion, CustomDecimalPoint) {
+  auto options = ConvertOptions::Defaults();
+  options.decimal_point = '/';
+
+  AssertConversion<FloatType, float>(float32(), {"1/5\n", "-1e10\n", "N/A\n"},
+                                     {{1.5, -1e10f, 0.}}, {{true, true, false}}, options);
+  AssertConversion<DoubleType, double>(float64(), {"1/5\n", "-1e10\n", "N/A\n"},
+                                       {{1.5, -1e10, 0.}}, {{true, true, false}},
+                                       options);
+  AssertConversionError(float32(), {"1.5\n"}, {0}, options);
+}
+
 TEST(BooleanConversion, Basics) {
   // XXX we may want to accept more bool-like values
   AssertConversion<BooleanType, bool>(boolean(), {"true,false\n", "1,0\n"},
@@ -350,6 +446,12 @@ TEST(BooleanConversion, CustomNulls) {
   auto options = ConvertOptions::Defaults();
   options.null_values = {"xxx", "zzz"};
 
+  AssertConversion<BooleanType, bool>(boolean(), {"\"true\",\"xxx\"\n"}, {{1}, {0}},
+                                      {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(boolean(), {"\"true\",\"xxx\"\n"}, {1}, options);
+
   AssertConversion<BooleanType, bool>(boolean(), {"true,xxx\n", "zzz,0\n"},
                                       {{true, false}, {false, false}},
                                       {{true, false}, {false, true}}, options);
@@ -365,6 +467,11 @@ TEST(Date32Conversion, Nulls) {
                                         {{false, true}});
 }
 
+TEST(Date32Conversion, Errors) {
+  AssertConversionError(date32(), {"1945-06-31\n"}, {0});
+  AssertConversionError(date32(), {"2020-13-01\n"}, {0});
+}
+
 TEST(Date64Conversion, Basics) {
   AssertConversion<Date64Type, int64_t>(date64(), {"1945-05-08\n", "2020-03-15\n"},
                                         {{-777945600000LL, 1584230400000LL}});
@@ -375,6 +482,63 @@ TEST(Date64Conversion, Nulls) {
                                         {{0, 1584230400000LL}}, {{false, true}});
 }
 
+TEST(Date64Conversion, Errors) {
+  AssertConversionError(date64(), {"1945-06-31\n"}, {0});
+  AssertConversionError(date64(), {"2020-13-01\n"}, {0});
+}
+
+TEST(Time32Conversion, Seconds) {
+  const auto type = time32(TimeUnit::SECOND);
+
+  AssertConversion<Time32Type, int32_t>(type, {"00:00\n", "00:00:00\n"}, {{0, 0}});
+  AssertConversion<Time32Type, int32_t>(type, {"01:23:45\n", "23:45:43\n"},
+                                        {{5025, 85543}});
+  AssertConversion<Time32Type, int32_t>(type, {"N/A\n", "23:59:59\n"}, {{0, 86399}},
+                                        {{false, true}});
+
+  AssertConversionError(type, {"24:00\n"}, {0});
+  AssertConversionError(type, {"23:59:60\n"}, {0});
+}
+
+TEST(Time32Conversion, Millis) {
+  const auto type = time32(TimeUnit::MILLI);
+
+  AssertConversion<Time32Type, int32_t>(type, {"00:00\n", "00:00:00\n"}, {{0, 0}});
+  AssertConversion<Time32Type, int32_t>(type, {"01:23:45.1\n", "23:45:43.789\n"},
+                                        {{5025100, 85543789}});
+  AssertConversion<Time32Type, int32_t>(type, {"N/A\n", "23:59:59.999\n"},
+                                        {{0, 86399999}}, {{false, true}});
+
+  AssertConversionError(type, {"24:00\n"}, {0});
+  AssertConversionError(type, {"23:59:60\n"}, {0});
+}
+
+TEST(Time64Conversion, Micros) {
+  const auto type = time64(TimeUnit::MICRO);
+
+  AssertConversion<Time64Type, int64_t>(type, {"00:00\n", "00:00:00\n"}, {{0LL, 0LL}});
+  AssertConversion<Time64Type, int64_t>(type, {"01:23:45.1\n", "23:45:43.456789\n"},
+                                        {{5025100000LL, 85543456789LL}});
+  AssertConversion<Time64Type, int64_t>(type, {"N/A\n", "23:59:59.999999\n"},
+                                        {{0, 86399999999LL}}, {{false, true}});
+
+  AssertConversionError(type, {"24:00\n"}, {0});
+  AssertConversionError(type, {"23:59:60\n"}, {0});
+}
+
+TEST(Time64Conversion, Nanos) {
+  const auto type = time64(TimeUnit::NANO);
+
+  AssertConversion<Time64Type, int64_t>(type, {"00:00\n", "00:00:00\n"}, {{0LL, 0LL}});
+  AssertConversion<Time64Type, int64_t>(type, {"01:23:45.1\n", "23:45:43.123456789\n"},
+                                        {{5025100000000LL, 85543123456789LL}});
+  AssertConversion<Time64Type, int64_t>(type, {"N/A\n", "23:59:59.999999999\n"},
+                                        {{0, 86399999999999LL}}, {{false, true}});
+
+  AssertConversionError(type, {"24:00\n"}, {0});
+  AssertConversionError(type, {"23:59:60\n"}, {0});
+}
+
 TEST(TimestampConversion, Basics) {
   auto type = timestamp(TimeUnit::SECOND);
 
@@ -385,12 +549,43 @@ TEST(TimestampConversion, Basics) {
                                            {"2018-11-13 17:11:10\n1900-02-28 12:34:56\n"},
                                            {{1542129070, -2203932304LL}});
 
+  // Zone offsets are not accepted
+  AssertConversionError(type,
+                        {"1970-01-01T00Z\n2000-02-29T00-0200\n"
+                         "3989-07-14T00+03:14\n1900-02-28 00-04:59\n"},
+                        {0});
+
   type = timestamp(TimeUnit::NANO);
   AssertConversion<TimestampType, int64_t>(
       type, {"1970-01-01\n2000-02-29\n1900-02-28\n"},
       {{0, 951782400000000000LL, -2203977600000000000LL}});
 }
 
+TEST(TimestampConversion, WithZoneOffset) {
+  auto type = timestamp(TimeUnit::SECOND, "UTC");
+
+  AssertConversion<TimestampType, int64_t>(
+      type,
+      {"1970-01-01T00Z\n2000-02-29T00-0200\n"
+       "3989-07-14T00+03:14\n1900-02-28 00-04:59\n"},
+      {{0, 951782400 + 7200, 63730281600LL - 11640, -2203977600LL + 17940}});
+
+  type = timestamp(TimeUnit::NANO, "UTC");
+  AssertConversion<TimestampType, int64_t>(
+      type,
+      {"1970-01-01T00Z\n"
+       "2000-02-29T00:00:00.123456789+0117\n"
+       "1900-02-28 00:00:00.123456789-01:00\n"},
+      {{0, 951782400000000000LL + 123456789LL - 4620000000000LL,
+        -2203977600000000000LL + 123456789LL + 3600000000000LL}});
+
+  // Local times are not accepted
+  AssertConversionError(type,
+                        {"1970-01-01T00\n2000-02-29T00\n"
+                         "3989-07-14T00\n1900-02-28 00\n"},
+                        {0});
+}
+
 TEST(TimestampConversion, Nulls) {
   auto type = timestamp(TimeUnit::MILLI);
   AssertConversion<TimestampType, int64_t>(
@@ -401,8 +596,14 @@ TEST(TimestampConversion, Nulls) {
 TEST(TimestampConversion, CustomNulls) {
   auto options = ConvertOptions::Defaults();
   options.null_values = {"xxx", "zzz"};
-
   auto type = timestamp(TimeUnit::MILLI);
+
+  AssertConversion<TimestampType, int64_t>(type, {"\"1970-01-01 00:01:00\",\"xxx\"\n"},
+                                           {{60000}, {0}}, {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(type, {"\"1970-01-01 00:01:00\",\"xxx\"\n"}, {1}, options);
+
   AssertConversion<TimestampType, int64_t>(type, {"1970-01-01 00:01:00,xxx,zzz\n"},
                                            {{60000}, {0}, {0}},
                                            {{true}, {false}, {false}}, options);
@@ -423,7 +624,43 @@ TEST(TimestampConversion, UserDefinedParsers) {
                                            {{86400000}, {172800000}}, options);
 }
 
-Decimal128 Dec128(util::string_view value) {
+TEST(TimestampConversion, UserDefinedParsersWithZone) {
+#ifdef __EMSCRIPTEN__
+  GTEST_SKIP() << "Test temporarily disabled due to emscripten bug "
+                  "https://github.com/emscripten-core/emscripten/issues/20467";
+#endif
+
+  auto options = ConvertOptions::Defaults();
+  auto type = timestamp(TimeUnit::SECOND, "America/Phoenix");
+
+  // Test a single parser
+  options.timestamp_parsers = {TimestampParser::MakeStrptime("%m/%d/%Y %z")};
+  if (internal::kStrptimeSupportsZone) {
+    AssertConversion<TimestampType, int64_t>(
+        type, {"01/02/1970 +0000,01/03/1970 +0000\n"}, {{86400}, {172800}}, options);
+  } else {
+    AssertConversionError(type, {"01/02/1970 +0000,01/03/1970 +0000\n"}, {0, 1}, options);
+  }
+
+  // Test multiple parsers
+  options.timestamp_parsers.push_back(TimestampParser::MakeISO8601());
+  if (internal::kStrptimeSupportsZone) {
+    AssertConversion<TimestampType, int64_t>(
+        type, {"01/02/1970 +0000,1970-01-03T00:00:00+0000\n"}, {{86400}, {172800}},
+        options);
+  } else {
+    AssertConversionError(type, {"01/02/1970 +0000,1970-01-03T00:00:00+0000\n"}, {0},
+                          options);
+  }
+
+  // Test errors
+  options.timestamp_parsers = {TimestampParser::MakeStrptime("%m/%d/%Y")};
+  AssertConversionError(type, {"01/02/1970,01/03/1970\n"}, {0, 1}, options);
+  options.timestamp_parsers.push_back(TimestampParser::MakeISO8601());
+  AssertConversionError(type, {"01/02/1970,1970-01-03T00:00:00+0000\n"}, {0}, options);
+}
+
+Decimal128 Dec128(std::string_view value) {
   Decimal128 dec;
   int32_t scale = 0;
   int32_t precision = 0;
@@ -433,41 +670,59 @@ Decimal128 Dec128(util::string_view value) {
 
 TEST(DecimalConversion, Basics) {
   AssertConversion<Decimal128Type, Decimal128>(
-      decimal(23, 2), {"12,34.5\n", "36.37,-1e5\n"},
+      decimal128(23, 2), {"12,34.5\n", "36.37,-1e5\n"},
       {{Dec128("12.00"), Dec128("36.37")}, {Dec128("34.50"), Dec128("-100000.00")}});
 }
 
 TEST(DecimalConversion, Nulls) {
   AssertConversion<Decimal128Type, Decimal128>(
-      decimal(14, 3), {"1.5,0.\n", ",-1e3\n"},
+      decimal128(14, 3), {"1.5,0.\n", ",-1e3\n"},
       {{Dec128("1.500"), Decimal128()}, {Decimal128(), Dec128("-1000.000")}},
       {{true, false}, {true, true}});
 
-  AssertConversionAllNulls<Decimal128Type, Decimal128>(decimal(14, 2));
+  AssertConversionAllNulls<Decimal128Type, Decimal128>(decimal128(14, 2));
 }
 
 TEST(DecimalConversion, CustomNulls) {
   auto options = ConvertOptions::Defaults();
   options.null_values = {"xxx", "zzz"};
 
+  AssertConversion<Decimal128Type, Decimal128>(decimal128(14, 3), {"\"1.5\",\"xxx\"\n"},
+                                               {{Dec128("1.500")}, {0}},
+                                               {{true}, {false}}, options);
+
+  options.quoted_strings_can_be_null = false;
+  AssertConversionError(decimal128(14, 3), {"\"1.5\",\"xxx\"\n"}, {1}, options);
+
   AssertConversion<Decimal128Type, Decimal128>(
-      decimal(14, 3), {"1.5,xxx\n", "zzz,-1e3\n"},
+      decimal128(14, 3), {"1.5,xxx\n", "zzz,-1e3\n"},
       {{Dec128("1.500"), Decimal128()}, {Decimal128(), Dec128("-1000.000")}},
       {{true, false}, {false, true}}, options);
 }
 
+TEST(DecimalConversion, CustomDecimalPoint) {
+  auto options = ConvertOptions::Defaults();
+  options.decimal_point = '/';
+
+  AssertConversion<Decimal128Type, Decimal128>(
+      decimal128(14, 3), {"1/5,0/\n", ",-1e3\n"},
+      {{Dec128("1.500"), Decimal128()}, {Decimal128(), Dec128("-1000.000")}},
+      {{true, false}, {true, true}}, options);
+  AssertConversionError(decimal128(14, 3), {"1.5\n"}, {0}, options);
+}
+
 TEST(DecimalConversion, Whitespace) {
   AssertConversion<Decimal128Type, Decimal128>(
-      decimal(5, 1), {" 12.00,34.5\n", " 0 ,-1e2 \n"},
+      decimal128(5, 1), {" 12.00,34.5\n", " 0 ,-1e2 \n"},
       {{Dec128("12.0"), Decimal128()}, {Dec128("34.5"), Dec128("-100.0")}});
 }
 
 TEST(DecimalConversion, OverflowFails) {
-  AssertConversionError(decimal(5, 0), {"1e6,0\n"}, {0});
+  AssertConversionError(decimal128(5, 0), {"1e6,0\n"}, {0});
 
-  AssertConversionError(decimal(5, 1), {"123.22\n"}, {0});
-  AssertConversionError(decimal(5, 1), {"12345.6\n"}, {0});
-  AssertConversionError(decimal(5, 1), {"1.61\n"}, {0});
+  AssertConversionError(decimal128(5, 1), {"123.22\n"}, {0});
+  AssertConversionError(decimal128(5, 1), {"12345.6\n"}, {0});
+  AssertConversionError(decimal128(5, 1), {"1.61\n"}, {0});
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -496,12 +751,18 @@ TYPED_TEST(TestNumericDictConverter, Nulls) {
   auto expected_indices = ArrayFromJSON(int32(), "[0, 1, null, 0]");
 
   AssertDictConversion("4\n5\nN/A\n4\n", expected_indices, expected_dict);
+  AssertDictConversion("\"4\"\n\"5\"\n\"N/A\"\n\"4\"\n", expected_indices, expected_dict);
 }
 
 TYPED_TEST(TestNumericDictConverter, Errors) {
   auto value_type = this->type();
   ASSERT_RAISES(Invalid, DictConversion(value_type, "xxx\n"));
 
+  ConvertOptions options = ConvertOptions::Defaults();
+
+  options.quoted_strings_can_be_null = false;
+  ASSERT_RAISES(Invalid, DictConversion(value_type, "\"N/A\"\n", -1, options));
+
   // Overflow
   if (is_integer(value_type->id())) {
     ASSERT_RAISES(Invalid, DictConversion(value_type, "99999999999999999999999\n"));
@@ -590,7 +851,7 @@ TEST(TestFixedSizeBinaryDictConverter, Errors) {
 }
 
 TEST(TestDecimalDictConverter, Basics) {
-  auto value_type = decimal(9, 3);
+  auto value_type = decimal128(9, 3);
 
   auto expected_dict = ArrayFromJSON(value_type, R"(["1.234", "456.789"])");
   auto expected_indices = ArrayFromJSON(int32(), "[0, 1, null, 1]");
@@ -599,8 +860,23 @@ TEST(TestDecimalDictConverter, Basics) {
                        expected_dict);
 }
 
+TEST(TestDecimalDictConverter, CustomDecimalPoint) {
+  auto value_type = decimal128(9, 3);
+
+  auto options = ConvertOptions::Defaults();
+  options.decimal_point = '\'';
+
+  auto expected_dict = ArrayFromJSON(value_type, R"(["1.234", "456.789"])");
+  auto expected_indices = ArrayFromJSON(int32(), "[0, 1, null, 1]");
+
+  AssertDictConversion("1'234\n456'789\nN/A\n4'56789e2\n", expected_indices,
+                       expected_dict, -1, options);
+
+  ASSERT_RAISES(Invalid, DictConversion(value_type, "1.234\n", -1, options));
+}
+
 TEST(TestDecimalDictConverter, Errors) {
-  auto value_type = decimal(9, 3);
+  auto value_type = decimal128(9, 3);
 
   // Overflow
   ASSERT_RAISES(Invalid, DictConversion(value_type, "1e10\n"));
