@@ -301,6 +301,51 @@ TEST_F(TestDecimal, TestCompare) {
                             outputs[5]);  // greater_than_or_equal_to
 }
 
+TEST_F(TestDecimal, TestNegative) {
+  // schema for input fields
+  constexpr int32_t precision = 3;
+  constexpr int32_t scale = 1;
+
+  auto decimal_type = std::make_shared<arrow::Decimal128Type>(precision, scale);
+
+  auto field_a = field("a", decimal_type);
+  auto schema = arrow::schema({field_a});
+
+  // build expressions
+  auto exprs = std::vector<ExpressionPtr>{
+      TreeExprBuilder::MakeExpression("negative", {field_a},
+                                      field("res_negative", decimal_type)),
+  };
+
+  // Build a projector for the expression.
+  std::shared_ptr<Projector> projector;
+  auto status = Projector::Make(schema, exprs, TestConfiguration(), &projector);
+  DCHECK_OK(status);
+
+  // Create a row-batch with some sample data
+  int num_records = 4;
+  auto validity = {true, true, true, true};
+  auto array_a = MakeArrowArrayDecimal(
+      decimal_type, MakeDecimalVector({"10.5", "-10.5", "-50.2", "50.2"}, scale),
+      validity);
+
+  // prepare input record batch
+  auto in_batch = arrow::RecordBatch::Make(schema, num_records, {array_a});
+
+  // Evaluate expression
+  arrow::ArrayVector outputs;
+  status = projector->Evaluate(*in_batch, pool_, &outputs);
+  DCHECK_OK(status);
+
+  // Validate results
+  // negative(x)
+  EXPECT_ARROW_ARRAY_EQUALS(
+      MakeArrowArrayDecimal(decimal_type,
+                            MakeDecimalVector({"-10.5", "10.5", "50.2", "-50.2"}, scale),
+                            validity),
+      outputs[0]);
+}
+
 // ARROW-9092: This test is conditionally disabled when building with LLVM 9
 // because it hangs.
 #if GANDIVA_LLVM_VERSION != 9
@@ -319,25 +364,25 @@ TEST_F(TestDecimal, TestRoundFunctions) {
   auto exprs = std::vector<ExpressionPtr>{
       TreeExprBuilder::MakeExpression("abs", {field_a}, field("res_abs", decimal_type)),
       TreeExprBuilder::MakeExpression("ceil", {field_a},
-                                      field("res_ceil", arrow::decimal(precision, 0))),
-      TreeExprBuilder::MakeExpression("floor", {field_a},
-                                      field("res_floor", arrow::decimal(precision, 0))),
-      TreeExprBuilder::MakeExpression("round", {field_a},
-                                      field("res_round", arrow::decimal(precision, 0))),
+                                      field("res_ceil", arrow::decimal128(precision, 0))),
       TreeExprBuilder::MakeExpression(
-          "truncate", {field_a}, field("res_truncate", arrow::decimal(precision, 0))),
+          "floor", {field_a}, field("res_floor", arrow::decimal128(precision, 0))),
+      TreeExprBuilder::MakeExpression(
+          "round", {field_a}, field("res_round", arrow::decimal128(precision, 0))),
+      TreeExprBuilder::MakeExpression(
+          "truncate", {field_a}, field("res_truncate", arrow::decimal128(precision, 0))),
 
       TreeExprBuilder::MakeExpression(
           TreeExprBuilder::MakeFunction("round",
                                         {TreeExprBuilder::MakeField(field_a), scale_1},
-                                        arrow::decimal(precision, 1)),
-          field("res_round_3", arrow::decimal(precision, 1))),
+                                        arrow::decimal128(precision, 1)),
+          field("res_round_3", arrow::decimal128(precision, 1))),
 
       TreeExprBuilder::MakeExpression(
           TreeExprBuilder::MakeFunction("truncate",
                                         {TreeExprBuilder::MakeField(field_a), scale_1},
-                                        arrow::decimal(precision, 1)),
-          field("res_truncate_3", arrow::decimal(precision, 1))),
+                                        arrow::decimal128(precision, 1)),
+          field("res_truncate_3", arrow::decimal128(precision, 1))),
   };
 
   // Build a projector for the expression.
@@ -371,38 +416,38 @@ TEST_F(TestDecimal, TestRoundFunctions) {
 
   // ceil(x)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 0),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 0),
                             MakeDecimalVector({"2", "2", "-1", "-1"}, 0), validity),
       outputs[1]);
 
   // floor(x)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 0),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 0),
                             MakeDecimalVector({"1", "1", "-2", "-2"}, 0), validity),
       outputs[2]);
 
   // round(x)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 0),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 0),
                             MakeDecimalVector({"1", "2", "-1", "-2"}, 0), validity),
       outputs[3]);
 
   // truncate(x)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 0),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 0),
                             MakeDecimalVector({"1", "1", "-1", "-1"}, 0), validity),
       outputs[4]);
 
   // round(x, 1)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 1),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 1),
                             MakeDecimalVector({"1.2", "1.6", "-1.2", "-1.6"}, 1),
                             validity),
       outputs[5]);
 
   // truncate(x, 1)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 1),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 1),
                             MakeDecimalVector({"1.2", "1.5", "-1.2", "-1.5"}, 1),
                             validity),
       outputs[6]);
@@ -487,7 +532,7 @@ TEST_F(TestDecimal, TestCastFunctions) {
 
   // castDECIMAL(decimal)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision, 1),
+      MakeArrowArrayDecimal(arrow::decimal128(precision, 1),
                             MakeDecimalVector({"1.2", "1.6", "-1.2", "-1.6"}, 1),
                             validity),
       outputs[4]);
@@ -1012,7 +1057,7 @@ TEST_F(TestDecimal, TestCastDecimalVarCharInvalidInput) {
   arrow::ArrayVector outputs_1;
   status = projector->Evaluate(*in_batch_1, pool_, &outputs_1);
   EXPECT_FALSE(status.ok()) << status.message();
-  EXPECT_TRUE(status.message().find("not a valid decimal number") != std::string::npos);
+  EXPECT_NE(status.message().find("not a valid decimal128 number"), std::string::npos);
 }
 
 TEST_F(TestDecimal, TestVarCharDecimalNestedCast) {
@@ -1112,17 +1157,83 @@ TEST_F(TestDecimal, TestCastDecimalOverflow) {
   // Validate results
   // castDECIMAL(decimal)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision_out, 1),
+      MakeArrowArrayDecimal(arrow::decimal128(precision_out, 1),
                             MakeDecimalVector({"1.2", "0.0", "-1.2", "-1.6"}, 1),
                             validity),
       outputs[0]);
 
   // castDECIMALNullOnOverflow(decimal)
   EXPECT_ARROW_ARRAY_EQUALS(
-      MakeArrowArrayDecimal(arrow::decimal(precision_out, 1),
+      MakeArrowArrayDecimal(arrow::decimal128(precision_out, 1),
                             MakeDecimalVector({"1.2", "1.6", "-1.2", "-1.6"}, 1),
                             {true, false, true, true}),
       outputs[1]);
 }
 
+TEST_F(TestDecimal, TestSha) {
+  // schema for input fields
+  const std::shared_ptr<arrow::DataType>& decimal_5_2 = arrow::decimal128(5, 2);
+  auto field_a = field("a", decimal_5_2);
+  auto schema = arrow::schema({field_a});
+
+  // output fields
+  auto res_0 = field("res0", utf8());
+  auto res_1 = field("res1", utf8());
+
+  // build expressions.
+  // hashSHA1(a)
+  auto node_a = TreeExprBuilder::MakeField(field_a);
+  auto hashSha1 = TreeExprBuilder::MakeFunction("hashSHA1", {node_a}, utf8());
+  auto expr_0 = TreeExprBuilder::MakeExpression(hashSha1, res_0);
+
+  auto hashSha256 = TreeExprBuilder::MakeFunction("hashSHA256", {node_a}, utf8());
+  auto expr_1 = TreeExprBuilder::MakeExpression(hashSha256, res_1);
+
+  // Build a projector for the expressions.
+  std::shared_ptr<Projector> projector;
+  auto status =
+      Projector::Make(schema, {expr_0, expr_1}, TestConfiguration(), &projector);
+  ASSERT_OK(status) << status.message();
+
+  // Create a row-batch with some sample data
+  int num_records = 3;
+  auto validity_array = {false, true, true};
+
+  auto array_dec = MakeArrowArrayDecimal(
+      decimal_5_2, MakeDecimalVector({"3.45", "0", "0.01"}, 2), validity_array);
+
+  // prepare input record batch
+  auto in_batch = arrow::RecordBatch::Make(schema, num_records, {array_dec});
+
+  // Evaluate expression
+  arrow::ArrayVector outputs;
+  status = projector->Evaluate(*in_batch, pool_, &outputs);
+  ASSERT_OK(status);
+
+  auto response = outputs.at(0);
+  EXPECT_EQ(response->null_count(), 0);
+  EXPECT_NE(response->GetScalar(0).ValueOrDie()->ToString(), "");
+
+  // Checks if the hash size in response is correct
+  const int sha1_hash_size = 40;
+  for (int i = 1; i < num_records; ++i) {
+    const auto& value_at_position = response->GetScalar(i).ValueOrDie()->ToString();
+
+    EXPECT_EQ(value_at_position.size(), sha1_hash_size);
+    EXPECT_NE(value_at_position, response->GetScalar(i - 1).ValueOrDie()->ToString());
+  }
+
+  response = outputs.at(1);
+  EXPECT_EQ(response->null_count(), 0);
+  EXPECT_NE(response->GetScalar(0).ValueOrDie()->ToString(), "");
+
+  // Checks if the hash size in response is correct
+  const int sha256_hash_size = 64;
+  for (int i = 1; i < num_records; ++i) {
+    const auto& value_at_position = response->GetScalar(i).ValueOrDie()->ToString();
+
+    EXPECT_EQ(value_at_position.size(), sha256_hash_size);
+    EXPECT_NE(value_at_position, response->GetScalar(i - 1).ValueOrDie()->ToString());
+  }
+}
 }  // namespace gandiva
