@@ -18,9 +18,10 @@
 # under the License.
 #
 
-set -e
+set -eu
 
 : ${SOURCE_DEFAULT:=1}
+: ${SOURCE_DOWNLOAD:=${SOURCE_DEFAULT}}
 : ${SOURCE_RAT:=${SOURCE_DEFAULT}}
 : ${SOURCE_UPLOAD:=${SOURCE_DEFAULT}}
 : ${SOURCE_PR:=${SOURCE_DEFAULT}}
@@ -37,15 +38,15 @@ fi
 version=$1
 rc=$2
 
-tag=apache-arrow-${version}
+tag=apache-arrow-${version}-rc${rc}
 maint_branch=maint-${version}
-release_candidate_branch="release-${version}-rc${rc_number}"
-tagrc=${tag}-rc${rc}
-rc_url="https://dist.apache.org/repos/dist/dev/arrow/${tagrc}"
+rc_branch="release-${version}-rc${rc}"
+rc_url="https://dist.apache.org/repos/dist/dev/arrow/${tag}"
 
 echo "Preparing source for tag ${tag}"
 
 : ${release_hash:=$(cd "${SOURCE_TOP_DIR}" && git rev-list --max-count=1 ${tag})}
+: ${GITHUB_REPOSITORY:=apache/arrow}
 
 if [ ${SOURCE_UPLOAD} -gt 0 ]; then
   if [ -z "$release_hash" ]; then
@@ -56,67 +57,64 @@ fi
 
 echo "Using commit $release_hash"
 
-tarball=${tag}.tar.gz
+tarball=apache-arrow-${version}.tar.gz
 
-rm -rf ${tag}
-# be conservative and use the release hash, even though git produces the same
-# archive (identical hashes) using the scm tag
-(cd "${SOURCE_TOP_DIR}" && \
-  git archive ${release_hash} --prefix ${tag}/) | \
-  tar xf -
-
-# Resolve all hard and symbolic links
-rm -rf ${tag}.tmp
-mv ${tag} ${tag}.tmp
-cp -R -L ${tag}.tmp ${tag}
-rm -rf ${tag}.tmp
-
-# Create a dummy .git/ directory to download the source files from GitHub with Source Link in C#.
-dummy_git=${tag}/csharp/dummy.git
-mkdir ${dummy_git}
-pushd ${dummy_git}
-echo ${release_hash} > HEAD
-echo '[remote "origin"] url = https://github.com/apache/arrow.git' >> config
-mkdir objects refs
-popd
-
-# Create new tarball from modified source directory
-tar czf ${tarball} ${tag}
-rm -rf ${tag}
+if [ ${SOURCE_DOWNLOAD} -gt 0 ]; then
+  # Wait for the release candidate workflow to finish before attempting
+  # to download the tarball from the GitHub Release.
+  . $SOURCE_DIR/utils-watch-gh-workflow.sh ${tag} "release_candidate.yml"
+  . $SOURCE_DIR/utils-watch-gh-workflow.sh ${tag} "csharp.yml"
+  rm -rf artifacts
+  gh release download ${tag} \
+    --dir artifacts \
+    --repo "${GITHUB_REPOSITORY}"
+fi
 
 if [ ${SOURCE_RAT} -gt 0 ]; then
-  "${SOURCE_DIR}/run-rat.sh" ${tarball}
+  "${SOURCE_DIR}/run-rat.sh" artifacts/${tarball}
 fi
 
-if type shasum >/dev/null 2>&1; then
-  sha256_generate="shasum -a 256"
-  sha512_generate="shasum -a 512"
-else
-  sha256_generate="sha256sum"
-  sha512_generate="sha512sum"
-fi
-
-
 if [ ${SOURCE_UPLOAD} -gt 0 ]; then
-  # sign the archive
-  gpg --armor --output ${tarball}.asc --detach-sig ${tarball}
-  ${sha256_generate} $tarball > ${tarball}.sha256
-  ${sha512_generate} $tarball > ${tarball}.sha512
+  rm -rf signed-artifacts
+  mkdir -p signed-artifacts
+
+  # sign the artifacts
+  for artifact in artifacts/*; do
+    case "${artifact}" in
+      *.sha256|*.sha512)
+        continue
+        ;;
+    esac
+    gpg \
+      --armor \
+      --detach-sig \
+      --output signed-artifacts/$(basename ${artifact}).asc \
+      ${artifact}
+  done
+
+  # Upload signed tarballs to GitHub Release
+  gh release upload ${tag} \
+     --repo "${GITHUB_REPOSITORY}" \
+     --verify-tag \
+     signed-artifacts/*
 
   # check out the arrow RC folder
   svn co --depth=empty https://dist.apache.org/repos/dist/dev/arrow tmp
 
   # add the release candidate for the tag
-  mkdir -p tmp/${tagrc}
+  mkdir -p tmp/${tag}
 
   # copy the rc tarball into the tmp dir
-  cp ${tarball}* tmp/${tagrc}
+  cp artifacts/${tarball}* tmp/${tag}
+  cp signed-artifacts/${tarball}.asc tmp/${tag}
 
   # commit to svn
-  svn add tmp/${tagrc}
-  svn ci -m "Apache Arrow ${version} RC${rc}" tmp/${tagrc}
+  svn add tmp/${tag}
+  svn ci -m "Apache Arrow ${version} RC${rc}" tmp/${tag}
 
   # clean up
+  rm -rf artifacts
+  rm -rf signed-artifacts
   rm -rf tmp
 
   echo "Success! The release candidate is available here:"
@@ -129,27 +127,37 @@ fi
 # Create Pull Request and Crossbow comment to run verify source tasks
 if [ ${SOURCE_PR} -gt 0 ]; then
   archery crossbow \
-    --github-token=${ARROW_GITHUB_API_TOKEN} \
     verify-release-candidate \
     --base-branch=${maint_branch} \
     --create-pr \
-    --head-branch=${release_candidate_branch} \
+    --head-branch=${rc_branch} \
     --pr-body="PR to verify Release Candidate" \
-    --pr-title="WIP: [Release] Verify ${release_candidate_branch}" \
-    --remote=https://github.com/apache/arrow \
+    --pr-title="WIP: [Release] Verify ${rc_branch}" \
+    --remote=https://github.com/${GITHUB_REPOSITORY} \
     --rc=${rc} \
     --verify-source \
     --version=${version}
 fi
 
 if [ ${SOURCE_VOTE} -gt 0 ]; then
+  gh_api_url="https://api.github.com/graphql"
+  curl_options=($gh_api_url)
+  curl_options+=(--header "Authorization: Bearer ${ARROW_GITHUB_API_TOKEN}")
+  curl_options+=(--data "{\"query\": \"query {search(query: \\\"repo:apache/arrow is:issue is:closed milestone:${version}\\\", type:ISSUE) {issueCount}}\"}")
+  n_resolved_issues=$(curl "${curl_options[@]}" | jq ".data.search.issueCount")
+  curl_options=(--header "Accept: application/vnd.github+json")
+  if [ -n "${ARROW_GITHUB_API_TOKEN:-}" ]; then
+    curl_options+=(--header "Authorization: Bearer ${ARROW_GITHUB_API_TOKEN}")
+  fi
+  curl_options+=(--get)
+  curl_options+=(--data "state=open")
+  curl_options+=(--data "head=apache:${rc_branch}")
+  curl_options+=(https://api.github.com/repos/${GITHUB_REPOSITORY}/pulls)
+  verify_pr_url=$(curl "${curl_options[@]}" | jq -r ".[0].html_url")
   echo "The following draft email has been created to send to the"
   echo "dev@arrow.apache.org mailing list"
   echo ""
   echo "---------------------------------------------------------"
-  jira_url="https://issues.apache.org/jira"
-  jql="project%20%3D%20ARROW%20AND%20status%20in%20%28Resolved%2C%20Closed%29%20AND%20fixVersion%20%3D%20${version}"
-  n_resolved_issues=$(curl "${jira_url}/rest/api/2/search/?jql=${jql}" | jq ".total")
   cat <<MAIL
 To: dev@arrow.apache.org
 Subject: [VOTE] Release Apache Arrow ${version} - RC${rc}
@@ -158,17 +166,19 @@ Hi,
 
 I would like to propose the following release candidate (RC${rc}) of Apache
 Arrow version ${version}. This is a release consisting of ${n_resolved_issues}
-resolved JIRA issues[1].
+resolved GitHub issues[1].
 
 This release candidate is based on commit:
 ${release_hash} [2]
 
 The source release rc${rc} is hosted at [3].
-The binary artifacts are hosted at [4][5][6][7][8][9][10][11].
-The changelog is located at [12].
+The binary artifacts are hosted at [4][5][6][7][8][9].
+The changelog is located at [10].
 
 Please download, verify checksums and signatures, run the unit tests,
-and vote on the release. See [13] for how to validate a release candidate.
+and vote on the release. See [11] for how to validate a release candidate.
+
+See also a verification result on GitHub pull request [12].
 
 The vote will be open for at least 72 hours.
 
@@ -176,19 +186,18 @@ The vote will be open for at least 72 hours.
 [ ] +0
 [ ] -1 Do not release this as Apache Arrow ${version} because...
 
-[1]: ${jira_url}/issues/?jql=${jql}
+[1]: https://github.com/apache/arrow/issues?q=is%3Aissue+milestone%3A${version}+is%3Aclosed
 [2]: https://github.com/apache/arrow/tree/${release_hash}
 [3]: ${rc_url}
-[4]: https://apache.jfrog.io/artifactory/arrow/almalinux-rc/
-[5]: https://apache.jfrog.io/artifactory/arrow/amazon-linux-rc/
-[6]: https://apache.jfrog.io/artifactory/arrow/centos-rc/
-[7]: https://apache.jfrog.io/artifactory/arrow/debian-rc/
-[8]: https://apache.jfrog.io/artifactory/arrow/java-rc/${version}-rc${rc}
-[9]: https://apache.jfrog.io/artifactory/arrow/nuget-rc/${version}-rc${rc}
-[10]: https://apache.jfrog.io/artifactory/arrow/python-rc/${version}-rc${rc}
-[11]: https://apache.jfrog.io/artifactory/arrow/ubuntu-rc/
-[12]: https://github.com/apache/arrow/blob/${release_hash}/CHANGELOG.md
-[13]: https://cwiki.apache.org/confluence/display/ARROW/How+to+Verify+Release+Candidates
+[4]: https://packages.apache.org/artifactory/arrow/almalinux-rc/
+[5]: https://packages.apache.org/artifactory/arrow/amazon-linux-rc/
+[6]: https://packages.apache.org/artifactory/arrow/centos-rc/
+[7]: https://packages.apache.org/artifactory/arrow/debian-rc/
+[8]: https://packages.apache.org/artifactory/arrow/ubuntu-rc/
+[9]: https://github.com/apache/arrow/releases/tag/apache-arrow-${version}-rc${rc}
+[10]: https://github.com/apache/arrow/blob/${release_hash}/CHANGELOG.md
+[11]: https://arrow.apache.org/docs/developers/release_verification.html
+[12]: ${verify_pr_url}
 MAIL
   echo "---------------------------------------------------------"
 fi
